# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import sys

from cStringIO import StringIO
from contextlib import contextmanager

from ci_libs import utils


@contextmanager
def capture_stderr(command, *args, **kwargs):
    out, sys.stderr = sys.stderr, StringIO()
    command(*args, **kwargs)
    sys.stderr.seek(0)
    yield sys.stderr.read()
    sys.stderr = out


@contextmanager
def capture_stdout(command, *args, **kwargs):
    out, sys.stdout = sys.stdout, StringIO()
    command(*args, **kwargs)
    sys.stdout.seek(0)
    yield sys.stdout.read()
    sys.stdout = out


def mock_load_config():
    # mock up settings needed by tests that normally come from config
    utils.AUTH_CONFIG = {"ci_url": "foo",
                         "auth_url": "test",
                         "auth_user": "test",
                         "auth_password": "test",
                         "auth_tenant_name": "test",
                         }
    utils.CI_URL = utils.AUTH_CONFIG['ci_url']


class CLITestCase(unittest.TestCase):
    '''Provides the necessary setup for doing CLI testing.'''
    def setUp(self):
        super(CLITestCase, self).setUp()
