# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import urllib2

from ci_utils.ticket_states import (
    TicketWorkflowStep,
    TicketWorkflowStepStatus,
)


class TicketApi(object):

    def __init__(self, url, ticket_id):
        self._url = url + '/api/v1'
        self._ticket_id = ticket_id

    def _patch(self, resource, data):
        req = urllib2.Request(resource)
        req.add_header('Content-Type', 'application/json')
        req.get_method = lambda: 'PATCH'
        urllib2.urlopen(req, json.dumps(data))

    def _post(self, url, data):
        req = urllib2.Request(url)
        req.add_header('Content-Type', 'application/json')
        resp = urllib2.urlopen(req, json.dumps(data))
        return resp.headers['location']

    def _get(self, url):
        resp = urllib2.urlopen(url)
        resp = resp.read()
        return json.loads(resp)

    def _patch_pkg_status(self, status):
        step = TicketWorkflowStep.PKG_BUILDING.value
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_pkg_waiting(self):
        self._patch_pkg_status(TicketWorkflowStepStatus.PKG_BUILDING_WAITING)

    def set_pkg_inprogress(self):
        self._patch_pkg_status(
            TicketWorkflowStepStatus.PKG_BUILDING_INPROGRESS)

    def set_pkg_complete(self, success=True):
        state = TicketWorkflowStepStatus.PKG_BUILDING_COMPLETED
        if not success:
            state = TicketWorkflowStepStatus.PKG_BUILDING_FAILED
        self._patch_pkg_status(state)

    def _patch_ticket_status(self, status):
        step = TicketWorkflowStep.IMAGE_BUILDING.value
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_image_waiting(self):
        self._patch_ticket_status(
            TicketWorkflowStepStatus.IMAGE_BUILDING_WAITING)

    def set_image_inprogress(self):
        self._patch_ticket_status(
            TicketWorkflowStepStatus.IMAGE_BUILDING_INPROGRESS)

    def set_image_complete(self, success=True):
        state = TicketWorkflowStepStatus.IMAGE_BUILDING_COMPLETED
        if not success:
            state = TicketWorkflowStepStatus.IMAGE_BUILDING_FAILED
        self._patch_ticket_status(state)

    def _patch_testing_status(self, status):
        step = TicketWorkflowStep.IMAGE_TESTING.value
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_testing_waiting(self):
        self._patch_testing_status(
            TicketWorkflowStepStatus.IMAGE_TESTING_WAITING)

    def set_testing_inprogress(self):
        self._patch_testing_status(
            TicketWorkflowStepStatus.IMAGE_TESTING_INPROGRESS)

    def set_testing_complete(self, success=True):
        state = TicketWorkflowStepStatus.IMAGE_TESTING_PASSED
        if not success:
            state = TicketWorkflowStepStatus.IMAGE_TESTING_FAILED
        self._patch_testing_status(state)

    def _patch_publishing_status(self, status):
        step = TicketWorkflowStep.PKG_PUBLISHING.value
        if status == TicketWorkflowStepStatus.PKG_PUBLISHING_COMPLETED:
            logging.info('publishing is final step, marking ticket completed')
            step = TicketWorkflowStep.COMPLETED.value
            status == TicketWorkflowStepStatus.COMPLETED
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_publishing_waiting(self):
        self._patch_publishing_status(
            TicketWorkflowStepStatus.PKG_PUBLISHING_WAITING)

    def set_publishing_inprogress(self):
        self._patch_publishing_status(
            TicketWorkflowStepStatus.PKG_PUBLISHING_INPROGRESS)

    def set_publishing_complete(self, success=True):
        state = TicketWorkflowStepStatus.PKG_PUBLISHING_COMPLETED
        if not success:
            state = TicketWorkflowStepStatus.PKG_PUBLISHING_FAILED
        self._patch_publishing_status(state)

    def add_artifact(self, name, artifact_url):
        url = self._url + '/ticketartifact/'
        data = {
            'type': 'LOGS',
            'name': name,
            'ticket': '/api/v1/ticket/' + self._ticket_id + '/',
            'reference': artifact_url,
        }
        try:
            return self._post(url, data)
        except:
            logging.exception('Unable to add %s to ticket-system' % name)

    def get_binaries(self):
        url = self._url + '/binarypackage/'
        data = self._get(url)
        return data['objects']
